'******************************************************************************************************
'*  Name    : P-RTOS-DEMO.BAS                                                                         *
'*  Author  : John Barrat                                                                             *
'*  Notice  : Copyright (c) 2011-12 John Barrat                                                       *
'*          : All Rights Reserved                                                                     *
'*  Date    : 24/10/2011                                                                              *
'*  Version : 1.2                                                                                     *
'*  Notes   : Demonstrates the primary functions offered in P-RTOS                                    *
'*          : For PDS the Hardware target is the PIC 18F252.                                         *
'*          : For Amicus the hardware target is the PIC 18F25K20                                     *
'******************************************************************************************************
'$Define Amicus                          ' Configure for Amicus18 board and compiler.

$define Project                         ' Tell Pre-Compiler this is the file to compile from.

$ifdef Amicus
    Device 18F25K20
    Reminders = Off                     ' silence unnecessary warnings
    Xtal = 16                           ' Standard Xtal in Amicus board
    PLL_Req = On                        ' and multiply up to 64mHz with the PLL
    Reminders = On                      ' un-silence reminders
$else
    Device 18F252                       ' Set up the device
    Reminders = Off                     ' silence unnecessary warnings
    Xtal = 10                           ' Max clock rate for device is 40mHz so set Xtal at 10mHz
    PLL_Req = On                        ' and multiply up to 40mHz with the PLL
    Reminders = On                      ' un-silence reminders
$endif

Optimiser_Level = 3                     ' Maximum optimisation
Bootloader On                           ' Make room for bootloader

$ifdef Amicus
    Symbol PPMPulse 255                 ' Gives approx 250uS period.
    Symbol ServoPosMax $800             ' Servo Range
    Symbol ServoOffset $F000            ' 
    Symbol ServoInc 128                 ' Increment for L-R steps
$else
    Symbol PPMPulse 250                 ' Gives approx 300uS period.
    Symbol ServoPosMax $1500            ' Servo Range
    Symbol ServoOffset $D800           
    Symbol ServoInc 84                  ' Increment for L-R steps
    $define TMR_V2                      ' Timer Macro Version for 18F252
    $define ADC_V1                      ' ADC Macro Version for 18F252
$endif

Symbol ServoMin ServoOffset
Symbol Servo1Third (ServoPosMax/3) + ServoOffset
Symbol ServoCentre (ServoPosMax/2) + ServoOffset
Symbol Servo2Third ((ServoPosMax/3) * 2) + ServoOffset
Symbol ServoFSD ServoPosMax + ServoOffset

Create_Coff = On                        ' if using ISIS or debugging in MPLAB include this declare

$define V_Info_Include P-RTOS_DemoVerInfo.Inc ; Tells VInfo to save to include file 

Include "P-RTOS_DemoVerInfo.inc"        ' Version number of program held in this file
Include "P-RTOS\P-RTOS Defines.inc"     ' Some initial Defines used by P-RTOS

'******************************************************************************************************
'* P-RTOS Configuration settings                                                                      *
'******************************************************************************************************

$define OSENABLE_DEBUG True             ' Asserts RB0 High on interrupt and resets to Low at end of ISR

$define OSTASKS_COUNT 12                ' Maximum Task count is 256
$define OSPRIO_COUNT 9                  ' Number of priority levels

$define OSENABLE_INTERRUPTS True        ' Enable interrupts - required for Timer services
$define OSENABLE_TIMER True             ' Enables timer service
$define OSENABLE_TIMEOUTS True          ' allow timeouts for events and counters
$define OSTICK_SOURCE T0                ' T0, T1, EXT
$ifdef Amicus
' Timer settings for 10mSec clock from 64MHz clock source - InterruptOn, 16Bit Timer, Internal Osc Divided 8 
    $define OSTIMER_CONFIG TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_8' Timer 0 settings  (Preload Value $B1F0)
    $define OSTIMER_PRELOAD $B1F0       ' Preload value   
$else
' Timer settings for 10mSec clock from 40MHz clock source - InterruptOn, 16Bit Timer, Internal Osc Divided 4 
    $define OSTIMER_CONFIG TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_4' Timer 0 settings  (Preload Value $9E78)
    $define OSTIMER_PRELOAD $9E78       ' Preload value   
$endif
$define OSTICK_CTR_SIZE 2               ' Size of OS Tick Counter (bytes) (must be 1, 2 or 4 max)
$define OSENABLE_CYCLIC_TIMERS True     ' allow cyclic timers to be created

$define OSENABLE_EVENTS True            ' Enables Events
$define OSEVENTS_COUNT 6                ' Max number of events
$define OSENABLE_MESSAGES True          ' Event Messages enabled
$define OSENABLE_SEMAPHORES True        ' Event Semaphores enabled
$define OSENABLE_EVENT_FLAGS False      ' Event Flags enabled
$define OSEVENT_FLAGS 1                 ' Max Event flags supported
                                        
$define OSENABLE_BUFFERED_SERIN         ' enable Buffered UART serial Input
$define OSENABLE_BUFFERED_SEROUT True   ' enable Buffered UART serial output     
$define OSSER_ERROR_EVENT False         ' enable Rx error event handling

$define OSISR_USER_HOOK ServoISR        ' Hook in the servo ISR

GoTo Start                              ' skip over the P-RTOS Routines

Include "P-RTOS\P-RTOS Main.inc"        ' Main P-RTOS Routines - includes other RTOS files P-RTOS Macros and P-RTOS Vars.
Include "ADC.inc"
'******************************************************************************************************
'* Some Variables used in the demo                                                                    *
'******************************************************************************************************
Dim Wrd As Word
Dim Ctr As Word
Dim Chr As Byte
Dim ServoPositions[4] As Word

'******************************************************************************************************
'* Aliases to the Task Control Block and the Event arrays                                             *
'******************************************************************************************************
Symbol T_Count    = OSTCBP(1)
Symbol T_LEDOut   = OSTCBP(2)
Symbol T_Menu     = OSTCBP(3)
Symbol T_OSCOut   = OSTCBP(4)
Symbol T_Delayed  = OSTCBP(5)
Symbol T_BinSem   = OSTCBP(6)
Symbol C_CycTmr   = OSTCBP(7)
Symbol T_RecTask  = OSTCBP(8)
Symbol T_ServoDrv = OSTCBP(9)
Symbol C_ServoTmr = OSTCBP(10)

Symbol E_LedCtrl  = OSECBP(1)
Symbol E_MsgTask  = OSECBP(2)
Symbol E_MsgRply  = OSECBP(3)
Symbol E_ServoDrv = OSECBP(4)

'******************************************************************************************************
'* Just a free running counter which signals E_LedCTrl each time it resets to 0                       *
'******************************************************************************************************
CountTsk:
Repeat
    Inc Ctr
    If Ctr = $1FFF Then
        Ctr = 0
        OSSignalBinSem E_LedCtrl
    EndIf
    OS_Yield
Forever

'******************************************************************************************************
'* Just a free running task which Toggles bit 2 of Port C - provides vibility on scope                *
'******************************************************************************************************
LEDOut:
Repeat
    Toggle PORTC.2
    Low PORTC.2   ' Botch to ensure bit always returns to 0.  
    OS_Yield
Forever

'******************************************************************************************************
'* Just a free running task which Toggles Bit 1 of Port C - provides vibility on scope  *                                                                 *
'******************************************************************************************************
OSCOut:
Repeat
    Toggle PORTC.1
    Low PORTC.1   ' Botch to ensure bit always returns to 0.
    OS_Yield
Forever

'******************************************************************************************************
'* This task switches On OSCOut task for 2 ticks stops it waits for 10 ticks and swaps to DelayedTask2*
'* and also raises priority at the same time                                                          *
'******************************************************************************************************
DelayedTask:
Repeat
    OSStartTask T_OSCOut
    OS_Delay 2
    OSStopTask T_OSCOut
    OS_Delay 10
    OS_Replace DelayedTask2, 3
Forever

'******************************************************************************************************
'* This task switches On OSCOut task for 1 tick stops it waits for 50 ticks and swaps to DelayedTask  *
'* and also lowers priority at the same time                                                          *
'******************************************************************************************************
DelayedTask2:                                                                                                
Repeat
    OSStartTask T_OSCOut
    OS_Delay 1
    OSStopTask T_OSCOut
    OS_Delay 50
    OS_Replace DelayedTask, 4
Forever

'******************************************************************************************************
'* This task waits a Binary Semaphore which is signalled by CountTsk.                                 *
'* When signalled it starts LEDOut leaves it on for 2 ticks and then stops the task.                  *
'******************************************************************************************************
BinSemTask:
Repeat
    OS_WaitBinSem E_LedCtrl, OSNO_TIMEOUT                                        
    OSStartTask T_LEDOut
    OS_Delay 2
    OSStopTask T_LEDOut
    Low PORTC.2            ' Botch to ensure bit always returns to 0.
Forever

'******************************************************************************************************
'* This test excercises the serial buffer in and out functions.                                       *
'* Within the capacity of the buffer and the baud rate you should be able to enter characters and see *
'* them appear on the Virtual terminal. Both input and output serial data is received/sent under      *
'* interrupt control                                                                                  *
'*                                                                                                    *
'* OS_WaitSem waits on serial data arriving in the buffer.                                            *
'* OS_WaitTXBuffer waits on the output buffer being empty before it sends more characters             *
'* ESC char swaps this task with MenuOutTask                                                          *
'******************************************************************************************************
CharOutTask:
Repeat
    OS_WaitTXBuffer
    HRSOut  "\rSerial Buffer Test...\r\r",_
            "Key presses will be echoed to the screen\r"              
    OS_WaitTXBuffer
    HRSOut  RtnMsg2 
Ch: 
    OS_WaitSem RxCharEv, OSNO_TIMEOUT
    HRSIn Chr
    If Chr = $1B Then
        OS_WaitTXBuffer
        HRSOut RtnMsg1
        OS_Replace MenuOutTask, 1
    Else
        HRSOut Chr
    EndIf
    GoTo Ch
Forever

'******************************************************************************************************
'* This test waits on a Key being pressed within a time limit. It also demonstrates how the timeout   *
'* can be extended beyond 255 ticks by calling it repeatedly.                                         *
'* Any key pressed will reset the timeout to maximum unless it's the Esc key which will Swap out this * 
'* task with MenuOutTask                                                                              *
'******************************************************************************************************
CharOutTOTask:
Dim i As Byte
Repeat
    OS_WaitTXBuffer
    HRSOut  "\rTimeout test...\r\r", RtnMsg2
ChRep:
    OS_WaitTXBuffer
    HRSOut   "Press a key within 10 Secs"
ChLp:
    OS_WaitSem RxCharEv, $FF
    If OSTimedOut = True Then
        Inc i
        Select i
        Case 1 
            HRSOut Rep $08\8," 7.5 Secs"
           GoTo ChLp
        Case 2
            HRSOut Rep $08\8,"5 Secs  "
           GoTo ChLp
        Case 3
            HRSOut Rep $08\8,"2.5 Secs"                                           
            GoTo ChLp
        Case 4
            HRSOut "\rTimed Out - restarting test\r\r"
            i = 0
            GoTo ChRep
        EndSelect
    Else
        i = 0
        HRSIn Chr
        If Chr = $1B Then
            OS_WaitTXBuffer     
            HRSOut RtnMsg1
            OS_Replace MenuOutTask, 1
        Else
            HRSOut Rep $08\8, " 10 Secs " 
        EndIf
    EndIf   
    GoTo ChLp
Forever

'******************************************************************************************************
'* This task sends a Message to another task which is waiting on that message.                        *
'* First it spawns a task to receive the message (T_RecTask), creates a Message Event (E_MsgTask) to  *
'* Signal the event to the waiting receiver task.  It also Creates a Binary Semaphore (E_MsgRply) to  *
'* which the Receive message task signals to the Send Message task that it has completed recceiving   *
'* the message and it is safe to write to the serial port.                                            *
'*                                                                                                    *
'* For the purposes of this test, the message is a String of up to 8 chars. On receipt of the message *
'* the Recieve message echos the message back to the serial port.                                     *
'*                                                                                                    *
'* If Esc key is pressed Sends Return to Main Menu message, Destroys the Receive message task and     *
'* swaps out to MenuOutTask.                                                                          *
'******************************************************************************************************
SendMessageTask:
Dim MsgStr[8] As Byte
Dim MsgAddr As Word
Repeat
    OS_WaitTXBuffer
    HRSOut "\rMessage Event Test...\r\r"
    OS_WaitTXBuffer
    HRSOut "Enter a string up to 8 chars followed by Enter\r"
    OS_WaitTXBuffer
    HRSOut RtnMsg2
    OSCreateTask T_RecTask, ReceiveMessageTask, 3
    OSCreateMsg E_MsgTask, 0
    OSCreateBinSem E_MsgRply, False
    OSStartTask T_RecTask
    i = 0
    HRSOut "\rEnter - "
MsgLoop:
    OS_WaitSem RxCharEv, OSNO_TIMEOUT    
    HRSIn Chr
    If Chr = $1B Then                               ' ESC char pressed
        OS_WaitTXBuffer
        HRSOut RtnMsg1
        OSDestroyTask T_RecTask                     ' kill the Receive Message task
        OS_Replace MenuOutTask, 1                   ' Swap out task back to menu
    Else
        If Chr = $0D Then SigMsg                    ' if terminated with Enter then message send message
        HRSOut Chr
        MsgStr[i] = Chr
        Inc i
        If i >= SizeOf MsgStr Then                  ' Filled buffer with message so send message
    SigMsg: OS_WaitTXBuffer
            If i = 0 Then GoTo MsgLoop
            HRSOut "\rSent - ", Str MsgStr\8 ,$0D
            MsgAddr = VarPtr(MsgStr)
            OSSignalMsg E_MsgTask, MsgAddr
            OS_WaitBinSem E_MsgRply, OSNO_TIMEOUT   ' Now wait until ReceiveMessage Task has completed
            Clear MsgStr                            ' Clear buffer
        Else
            GoTo MsgLoop 
        EndIf
    EndIf
Forever

ReceiveMessageTask:
Dim MsgPtr As Word
Dim OSFSR0 As FSR0L.Word
Dim j As Byte
Repeat
    OS_WaitMsg E_MsgTask, OSNO_TIMEOUT, MsgPtr
    HRSOut "Received - "
    i = 0
    OSFSR0 = MsgPtr                                 ' Load FSR0 with address of Message
    Repeat                                          ' Unload message....
        Chr = POSTINC0                              ' Get byte into Chr and increment pointer
        If Chr = 0 Then Break                       ' Break if Nul
        HRSOut Chr                                  ' Output the byte
        Inc i                                       ' Inc byte ctr
    Until i = 8                                     ' until message unloaded
    HRSOut $0D
    OSSignalBinSem E_MsgRply                        ' tell SendMessage task you have done all printing    
Forever

'******************************************************************************************************
'* This task reports the current value of the Tick Counter every second (Tick frquency is 100 Hz)     *
'* It does not use any keyboard input so it spawns a new CheckInput task to monitor the keyboard.     *
'* This task uses the same TCB as the ReceiveMessagetask which is spawned by the SendMessageTask.     *
'*                                                                                                    *
'* OS_ReplaceTask is a yielding function so cannot be used by the CheckInput to return to the main    *
'* MenuOutTask.  So all it does is monitor the keyboard and leave the value in Chr. The SendMessage   *
'* task checks the Chr value for ESC key. If detected, it destroys the CheckInput task, send the      *
'* "Returning to Main Menu" message and then replaces itself with MenuOutTask.                        *
'******************************************************************************************************   
ShowTickCount:
Dim TickCount As Word
Repeat    
    HRSOut "\rShow P-RTOS Tick Count\r"
    OSCreateTask T_RecTask, CheckInput, 1    
    HRSOut RtnMsg2
    Chr = 0
    OSStartTask T_RecTask
StLp:
    TickCount = OSGetTicks
    HRSOut Dec5 TickCount
    OS_Delay 100
    If Chr = $1B Then
        OSDestroyTask T_RecTask
        HRSOut RtnMsg1 
        OS_Replace MenuOutTask, 1
    Else
        HRSOut Rep $8 \5
    EndIf
    GoTo StLp
Forever

CheckInput:
Repeat
    OS_HRSIn(OSNO_TIMEOUT, Chr)
Forever

'******************************************************************************************************   
' This task exercises the servos.  On entering it sets up some initial values for each servo and      *
' starts the Cyclic timer which provides the basic 20mSec timebase for the servo update. The task     *
' presents a menu for manual control of individual servos from the keyboard or variable resistor auto-*
' matic control of all 4 servos driving between +/- 90 Deg. Interrupts are disabled when updating the *
' servo positions to prevent the ServoDrv task reading a partially updated value.                     *
'******************************************************************************************************   

SetServoValTask:
Dim ServoPos As Word
Dim ServoModes As Byte                 ' 
    Dim Dir As ServoModes              ' bits 0-3 used for Dir
    Dim AbsPosFlg As ServoModes.4
    Dim PPMMode As ServoModes.5
    Dim Up As ServoModes.6
Dim ServoNo As Byte
Dim DirMsk As Byte
Dim TmpW As Word

' Set up some initial servo positions

Repeat
    OS_HRSOut "\rDrive or step a servo position\r\r"    
    OS_HRSOut "P    Toggles Pulse Position output on PortB.4\r" 
    OS_HRSOut "S0-3 Select servo number 0 to 3 for manual control\r"
    $ifdef Amicus
        OS_HRSOut "0-N  Enter a number between 0 and 2048 followed by Enter\r"
    $else
        OS_HRSOut "0-N  Enter a number between 0 and 5376 followed by Enter\r"
    $endif
    OS_HRSOut "L-R  Steps servo by approx 6 deg\r" 
    OS_HRSOut "M    Toggles servo position control from ADC0\r"
    OS_HRSOut "A    Toggles automatic drive to all servos\r"
    OS_HRSOut RtnMsg2
    Chr = 0
    ServoPositions[0] = ServoMin    ' Full Scale Anticlockwise
    ServoPositions[1] = Servo1Third ' 1/3 FSD
    ServoPositions[2] = Servo2Third ' 2/3 FSD
    ServoPositions[3] = ServoFSD    ' Full Scale Clockwise
    OSStartCycTmr C_ServoTmr       
    ServoNo = 0
    ServoPos = ServoPositions[ServoNo]
    ServoPos = ServoPos - $D800
    AbsPosFlg = False
SVLp:
    OS_HRSIn(OSNO_TIMEOUT, Chr)
    Select Chr
    Case $0D
        GoSub UpdateServos
        HRSOut $0D
        AbsPosFlg = False            
    Case $30 To $39
        If AbsPosFlg = False Then
            ServoPos = 0
            AbsPosFlg = True
        EndIf
        HRSOut Chr
        Chr = Chr - $30
        ServoPos = ServoPos * 10
        ServoPos = ServoPos + Chr
    Case "A", "a"
        j = OSGetStateTask T_RecTask
        If j > $10 Then
            DirMsk = $1
            Dir = $1
            ServoNo = 0
            OSCreateTask T_RecTask, DriveAllServos, 1
            OSStartTask T_RecTask
            HRSOut "Servos on Auto\r"
        Else
            OSStopTask T_RecTask
            OSDestroyTask T_RecTask
            HRSOut "Servos on Manual\r"
        EndIf        
    Case "L", "l"
        If AbsPosFlg = False Then
            If ServoPos > 0 Then ServoPos = ServoPos - ServoInc
            GoSub UpdateServos
        EndIf
    Case "M", "m"
        j = OSGetStateTask T_RecTask
        If j > $10 Then
            OSCreateTask T_RecTask, SetManualServoTask, 1
            OSStartTask T_RecTask
            HRSOut "Pot controls Servos\r"
        Else
            OSStopTask T_RecTask
            OSDestroyTask T_RecTask
            HRSOut "Pot control Off\r"
        EndIf     
    Case "P", "p"
        PPMMode = ~ PPMMode
        If PPMMode = True Then 
            HRSOut "PPM Output on\r"
        Else
            HRSOut "PPM Output off\r"
        EndIf
    Case "R", "r"
        If AbsPosFlg = False Then
            ServoPos = ServoPos + ServoInc
            GoSub UpdateServos
        EndIf
    Case "S", "s"
        OS_HRSIn(OSNO_TIMEOUT, Chr)
        If Chr > $2F And Chr < $34 Then 
            ServoNo = Chr - $30
            HRSOut "Servo ", Chr, " Selected\r"
        EndIf
    Case $1B
        GIE = 0
        j = OSGetStateTask T_RecTask      ' see if servos running
        If j <= $10 Then OSStopTask T_RecTask ' and stop if running.
        EndIf
        ServoPositions[0] = ServoCentre   ' Reset to 1/2 FSD 0Deg
        ServoPositions[1] = ServoCentre   ' 
        ServoPositions[2] = ServoCentre   ' 
        ServoPositions[3] = ServoCentre   '
        GIE = 1                           ' re-enable interrupts
        OS_Delay 50                       ' allow time for the servos to return to centre
        OSStopCycTmr C_ServoTmr
        HRSOut RtnMsg1                                                                      
        OSDestroyTask T_RecTask
        OS_Replace MenuOutTask, 2               
    End Select        
    GoTo SVLp
Forever

UpdateServos:
    If ServoPos > ServoPosMax Then ServoPos = ServoPosMax
    GIE = 0       
    ServoPositions[ServoNo] = ServoPos + ServoOffset      
    GIE = 1
Return

'*******************************************************************************************************
'* Cycles the servos between their max and min positions.                                              * 
'*******************************************************************************************************
DriveAllServos:
Repeat
    Repeat
        GIE = 0
        ServoPos = ServoPositions[ServoNo]      ' get the servo position
        GIE = 1
        ServoPos = ServoPos - ServoOffset       ' adjust its baseline
        If Dir & DirMsk > 0 Then
            Up = True
        Else
            Up = False
        EndIf
        If Up = True Then
            If ServoPos > ServoPosMax -$20 Then ' Close to Max so start oounting down now?
                Dir = Dir & ~DirMsk             ' Save new state for this servo
                Up = False
            EndIf
            ServoPos = ServoPos + $20           ' Step servo position up
        Else
            If ServoPos <= $20 Then             ' close to minimum
                Dir = Dir | DirMsk              ' ssve new direction for this servo
                Up = True
            EndIf
            ServoPos = ServoPos - $20           ' Step servo poistion down
        EndIf
        GoSub UpdateServos                      ' update servo
        Inc ServoNo                             ' on to next servo
        DirMsk = DirMsk << 1                    ' set up new servo mask
    Until ServoNo > 3                           ' loop until all are updated
    ServoNo = 0                                 ' reset servo number
    DirMsk = $1                                 ' and mask
    OS_Delay 2                                  ' Wait a bit before starting again
Forever

'*******************************************************************************************************
'* This task uses the voltage on ADC Channel 0 to set the Servo position.  Note the delay of 1 OS tick *
'* to allow the ADC to acquire the voltage on channel 0. This could have been achieved by placing this *
'* code at the end of the ServoDrv task rather than as a separate task.                                *   
'*******************************************************************************************************
SetManualServoTask:
Repeat                                                                                                            
    TmpW = ReadADC(0)                           ' Get the current ADC value
    ServoPos = TmpW * 5                         ' Scale it and make this the new servo position
    GoSub UpdateServos                          ' Update the servo positions array
    ConvertADC                                  ' Initiate the next ADC conversion
    OS_Delay 1                                  ' Wait for conversion to take place
Forever 

'*******************************************************************************************************
'* This task provides the menu and invokes the other tasks.  It lists all the options in a menu then   * 
'* Waits for a command.  When a key is pressed the task continues, it reads in the key into Chr and as *
'* long as it is numeric or alphabetic echos it. It then selects the action based on the key pressed   *
'* Actions 1 though 4 control other continuous tasks while the others replace MenuOutTask with the     *
'* appropriate task. Replacing tasks means you can keep the number of TCBs required down by utilising  *
'* an existing TCB.                                                                                    * 
'*                                                                                                     *
'* See comments with each Case statement for details on each function.                                 *
'*******************************************************************************************************
MenuOutTask:
Repeat
    Chr = 0
    Wrd = OSVersion
    OS_WaitTXBuffer
    HRSOut  "P-RTOS Test Program ver ", V_InfoVersionStr, $0D
    OS_WaitTXBuffer
    HRSOut "Testing P-RTOS Ver: ", _
            Hex1 Wrd.HighByte >> 4,".", Hex1 Wrd.HighByte & $0F, ".", _
            Hex1 Wrd.LowByte >> 4, ".", Hex1 Wrd.LowByte & $0F, $0D, $0D       
    OS_HRSOut "1    Toggle Delayed Task\r"
    OS_HRSOut "2    Toggle Binary Semaphore Task\r"
    OS_HRSOut "3    Toggle Cyclic Timer\r"
    OS_HRSOut "4    Reset Cyclic Timer\r"
    OS_HRSOut "5    Show Tick Count\r"
    OS_HRSOut "6    Reset Tick Count\r"
    OS_HRSOut "7    Message Events Test\r"
    OS_HRSOut "8    Serial Buffer Test\r"
    OS_HRSOut "9    Event Timeout Test\r"
    OS_HRSOut "A    Toggle Servo Test\r\r"
    OS_HRSOut "Command>  "
ch1:    
    OS_HRSIn(OSNO_TIMEOUT, Chr)
    If Chr < $30 Then GoTo ch1
    HRSOut $08, Chr
'    GIE = 0
    Select Chr
    
    '***************************************************************************************************
    '* Read the state of the T_Delayed (Delayedtask). If it's running it could have one of 2 values:   * 
    '* OSTCB_TASK_ELIGABLE ($00) or OSTCB_TASK_DELAYED ($10) anything above that means its stopped.    *
    '* The task is started or stopped according to its current state.                                  *
    '***************************************************************************************************               
    Case "1"
        j = OSGetStateTask T_Delayed
        If j > $10 Then
            OSStartTask T_Delayed
        Else
            OSStopTask T_Delayed
        EndIf
        
    '***************************************************************************************************
    ' Similar to Case 1 but tests and controls the running state of T_BinSem (BinSemTask)              *    
    '***************************************************************************************************           
    Case "2"
        j = OSGetStateTask T_BinSem
        If j > $20 Then
             OSStartTask T_BinSem
        Else
            OSStopTask T_BinSem
        EndIf 
                  
    '***************************************************************************************************
    ' Similar to Case 1 but tests and controls the running state of CycTmr (CycTmrTest)                *    
    '***************************************************************************************************           
    Case "3" 
        j = OSCycTmrRunning C_CycTmr    ' Compiler throws error if this is used 
        If j = True Then
            OSStopCycTmr C_CycTmr            
        Else
            OSStartCycTmr C_CycTmr       
        EndIf     
              
    '***************************************************************************************************
    ' Issues a reset command to the cyclic timer CycTmr (CycTmrTest)                                   *    
    '***************************************************************************************************           
    Case "4"
        OSResetCycTmr C_CycTmr
     
    '***************************************************************************************************
    ' Swaps itself with ShowTickCount                                                                  *
    '***************************************************************************************************           
    Case "5"
        OS_Replace ShowTickCount, 1

    '***************************************************************************************************
    ' Sets the Tick Counter back to 0 and Swaps itself with ShowTickCount                              *
    '***************************************************************************************************
    Case "6"
        OSSetTicks 0
        OS_Replace ShowTickCount, 1

    '***************************************************************************************************
    ' Swaps itself with SendMessageTask                                                                *
    '***************************************************************************************************           
    Case "7" 
        OS_Replace SendMessageTask, 1

    '***************************************************************************************************
    ' Swaps itself with CharOutTask                                                                    *
    '***************************************************************************************************           
    Case "8"
        OS_Replace CharOutTask, 1

    '***************************************************************************************************
    ' Swaps itself with CharOutTOTask                                                                  *
    '***************************************************************************************************           
    Case "9"
        OS_Replace CharOutTOTask, 1
    
    '***************************************************************************************************
    ' Starts the Servo drivers and routines -                                                          *
    '***************************************************************************************************           
    Case "A", "a"
        OS_Replace SetServoValTask, 2 
               
    Case Else
    EndSelect
'    GIE = 1
    GoTo ch1
Forever

'*******************************************************************************************************
'* This routine is fired by the Cyclic timer.  It simply toggles a port pin long enough to be seen on  * 
'* ISIS digital scope. Note it's a routine not a task so doesnt have to yield but can't be interrupted *
'* In use these routines should be kept short as they only return to the scheduler when the complete   *
'*******************************************************************************************************
CycTmrTest:
    Toggle PORTC.2
    DelayUS 12              '  Cause a delay so you can see spike on screen 
    Toggle PORTC.2
Return

'*******************************************************************************************************
'* These routines generate RC Servo pulses for 4 servos on bits 0 to 3 of portB.                       *
'* A Cyclic timer (ServoTmr) determines the refresh rate of the servos (Typically 20mSec) This starts  *
'* up the main ServoTask (SeroDrv) which sets the pulse width for each servo and sets the appropriate  *
'* bit high for that servo. A hardware timer (Timer3) determines the pulse width for each servo. On    *
'* overflow the timer will trigger an interrupt which will be serviced by ServoISR. The ISR resets the *
'* appropriate bit for that servo and signals the event to the ServoDrv task.  The servo driver task   *
'* repeats the cycle until all servos have been serviced when it stops.                                *
'* The whole sequence is restarted when the Cyclic timer next runs.                                    *
'*******************************************************************************************************

Dim Tw As Word
Dim ClrMsk As Byte
Dim idx As Byte
Dim SetMsk As Byte

ServoDrv:
Repeat
    WriteTimer2 PPMPulse
    SetMsk = %0000001                        ' set mask initial value
    ClrMsk = ~ SetMsk                        ' set Clear mask initial value
    idx = 0                                  ' and index to first servo position
    Repeat
        Tw = ServoPositions[idx]             ' Get Servo position 
        WriteTimer3 Tw                       ' Set Servo pulse width
        StartTimer3                          ' Start Servo Pulse Timer
        If PPMMode = True Then
            Set PORTB.4                      ' Set PPM pulse
            StartTimer2                      ' Start PPM width timer
        EndIf
        PORTB = PORTB | SetMsk               ' Write to the port to set servo pulse high
        OS_WaitBinSem E_ServoDrv,1           ' Wait on server pulse timeout
        If OSTimedOut = True Then SvError    ' handle error
        SetMsk = SetMsk <<1                  ' Update Set Bit Mask 
        ClrMsk = ~ SetMsk                    ' Update Clear Bit mask
        Inc idx                              ' on to next servo
    Until idx > 3                            ' until all servos serviced
        If PPMMode = True Then
            Set PORTB.4
            StartTimer2
        EndIf
SvError:
    OS_Stop                                  ' stop and wait to be re triggered.
Forever

ServoISR:
High_Int_Sub_Start
    If PIR1.1 = 1 Then                       ' if Timer2 Imnterrupt flag set
        Clear PORTB.4                        ' clear PPM pulse down
        CloseTimer2                          ' Stop timer
        Clear PIR1.1                         ' clear the interrupt flag
    EndIf
    If PIR2.1 = 1 Then                       ' if Timer3 Interrupt Flag set
        PORTB = PORTB & ClrMsk               ' clear Servo pulse to low
        CloseTimer3                          ' stop timer
        OSQueue_Event E_ServoDrv | OSBINSEMEVNT ' Add event to the event queue (Signal timeout for servo)
        Clear PIR2.1                         ' Clear the interrupt flag
    EndIf
High_Int_Sub_End
GoTo OSUSER_ISR_RTN

ServoTmr:
    OSStartTask T_ServoDrv                   ' Start the Servo driver.    
Return

'-----Start---------------------------------------------------------------------------------------------
Start:
TRISA = %000001             ' All Port A Bit 0 input, all other bits Outputs
TRISB = %00000000           ' All Port B Outputs
TRISC = %11000000           ' Set port C to output except TxRX

Hserial_Baud = 9600         ' Set baud rate for USART 1
Ctr = $00                   ' reset ctr

OSInit                      ' Initialise RTOS
' Create Round robin tasks - these must all be on the same priority and below all other priorities
OSCreateTask T_Count, CountTsk, 5
OSCreateTask T_LEDOut, LEDOut, 5
OSCreateTask T_OSCOut, OSCOut, 5

' Create Delayed and semaphored tasks - these take priority above round robin task.
OSCreateTask T_Delayed, DelayedTask, 4
OSCreateTask T_BinSem, BinSemTask, 3
OSCreateTask T_Menu, MenuOutTask, 1
OSCreateTask T_ServoDrv, ServoDrv, 2

' Create Cyclic Timers
OSCreateCycTmr CycTmrTest, C_CycTmr, 20,5, OSCT_DONT_START_CYCTMR
OSCreateCycTmr ServoTmr, C_ServoTmr, 10,2, OSCT_DONT_START_CYCTMR ' 20mSec timer to start servo cycle

' Create Semaphores
OSCreateBinSem E_LedCtrl, 0
OSCreateBinSem E_ServoDrv, 0

' Set up Timers 2 & 3 used for Servo driver
' Timer 2 determines the fixed pulse width for PPM signal
' Timer 3 determines the pulse width for each servo - leave stopped.
$ifdef Amicus
    OpenTimer2 TIMER_INT_ON & T2_PS_1_1 & T2_POST_1_16 | TMR_STOPPED
    OpenTimer3 TIMER_INT_ON & T3_16BIT_RW & T3_SOURCE_INT & T3_PS_1_8 & T3_SYNC_EXT_OFF & T1_SOURCE_CCP | TMR_STOPPED
    OpenADC (ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_20_TAD, ADC_CH0 & ADC_REF_VDD_VSS, ADC_1ANA)     ' Configure the ADC
$else
    T2CON = %01011000           ' Open Timer2 but dont run it.
    'There is a problem with the OpenTimer3 macro when used with 18F252
    'OpenTimer3 (T3_16BIT_RW & T3_SOURCE_INT & T3_PS_1_8 & T3_SYNC_EXT_OFF & T1_SOURCE_CCP) | TMR_STOPPED
    T3CON = %10011100           ' Open Timer3 but dont run it.
    OpenADC (ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_1ANA_0REF, ADC_CH0)    ' Configure the ADC
    ADC_INT_DISABLE
$endif
    
' Start the tasks.
OSStartTask T_Count         ' This task just counts and signals a semaphore once per complete rotation
OSStartTask T_Menu          ' Start up the menu

Repeat
    OSSched                 ' run scheduler continuously
Forever

RtnMsg1: CData "\rReturning to Main Menu \r\r" ,0
RtnMsg2: CData "ESC  Return to Main Menu\r",0

